package
Version:
v1.2.8
Opens a new window with list of versions in this module.
Published: May 10, 2025
License: Apache-2.0
Opens a new window with license information.
Imports: 2
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
-
func Read(mem Memory, fieldName string, offset, byteCount uint32) []byte
-
func ReadString(mem Memory, fieldName string, offset, byteCount uint32) string
-
func Uint32FromInt(v int) (uint32, error)
-
func Write(mem Memory, fieldName string, offset uint32, data []byte)
-
type Memory
Read is like api.Memory except that it panics if the offset and byteCount are out of range.
ReadString is a convenience function that casts requireRead
Uint32FromInt safely converts an int to uint32, checking for negative values and overflow
Write is like api.Memory except that it panics if the offset and byteCount are out of range.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.